#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_file.interfaces import IVideo, IThumbnail, IAudio
from pyams_form.interfaces.form import IWidgetsPrefixViewletsManager
from pyams_media.interfaces import IMediaConversionUtility, VIDEO_FRAME_SIZE, IMediaConverter, IMediaConversions
from pyams_skin.interfaces.viewlet import IContextActions
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.layer import IAdminLayer

# import packages
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_template.template import template_config
from pyams_utils.interfaces import MANAGE_PERMISSION
from pyams_utils.registry import query_utility
from pyams_viewlet.viewlet import viewlet_config, Viewlet
from pyams_zmi.form import AdminDialogDisplayForm
from z3c.form import field
from zope.interface import Interface

from pyams_media import _


#
# Video conversions
#

@viewlet_config(name='video.conversions.action', context=IVideo, layer=IPyAMSLayer, view=Interface,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=15)
class VideoConversionsAction(ToolbarMenuItem):
    """Video conversions action"""

    label = _("Video conversions...")
    label_css_class = 'fa fa-fw fa-compress'

    url = 'video-conversions.html'
    modal_target = True


@pagelet_config(name='video-conversions.html', context=IVideo, layer=IPyAMSLayer, permission=MANAGE_PERMISSION)
class VideoConversionsDisplayForm(AdminDialogDisplayForm):
    """Video conversions display form"""

    legend = _("Selected video conversions")
    icon_css_class = 'fa fa-fw fa-compress'

    fields = field.Fields(Interface)

    @property
    def title(self):
        return self.context.title or self.context.filename


@viewlet_config(name='video-conversions-prefix', context=IVideo, layer=IAdminLayer,
                view=VideoConversionsDisplayForm, manager=IWidgetsPrefixViewletsManager)
@template_config(template='templates/video-conversions.pt')
class VideoConversionsViewletsPrefix(Viewlet):
    """Video conversions display form viewlets prefix"""

    @property
    def formats(self):
        utility = query_utility(IMediaConversionUtility)
        if utility is not None:
            for format in utility.video_frame_size:
                yield format, VIDEO_FRAME_SIZE.get(format)

    @property
    def converters(self):
        utility = query_utility(IMediaConversionUtility)
        if utility is not None:
            for converter_name in utility.video_formats:
                converter = query_utility(IMediaConverter, name=converter_name)
                if converter is not None:
                    yield converter_name, converter

    @property
    def original_frame_size(self):
        return IThumbnail(self.context).get_image_size()

    def has_conversion(self, converter, frame_size):
        conversions = IMediaConversions(self.context)
        name = 'video{width}.{extension}'.format(width='-{0}'.format(frame_size) if frame_size else '',
                                                 extension=converter.format)
        return name in conversions

    def get_conversion(self, converter, frame_size):
        conversions = IMediaConversions(self.context)
        name = 'video{width}.{extension}'.format(width='-{0}'.format(frame_size) if frame_size else '',
                                                 extension=converter.format)
        return conversions.get(name)


#
# Audio conversions
#

@viewlet_config(name='audio.conversions.action', context=IAudio, layer=IPyAMSLayer, view=Interface,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=15)
class AudioConversionsAction(ToolbarMenuItem):
    """Audio conversions action"""

    label = _("Audio conversions...")
    label_css_class = 'fa fa-fw fa-compress'

    url = 'audio-conversions.html'
    modal_target = True


@pagelet_config(name='audio-conversions.html', context=IAudio, layer=IPyAMSLayer, permission=MANAGE_PERMISSION)
class AudioConversionsDisplayForm(AdminDialogDisplayForm):
    """Audio conversions display form"""

    legend = _("Selected audio conversions")
    icon_css_class = 'fa fa-fw fa-compress'

    fields = field.Fields(Interface)

    @property
    def title(self):
        return self.context.title or self.context.filename


@viewlet_config(name='audio-conversions-prefix', context=IAudio, layer=IAdminLayer,
                view=AudioConversionsDisplayForm, manager=IWidgetsPrefixViewletsManager)
@template_config(template='templates/audio-conversions.pt')
class AudioConversionsViewletsPrefix(Viewlet):
    """Audio conversions display form viewlets prefix"""

    @property
    def converters(self):
        utility = query_utility(IMediaConversionUtility)
        if utility is not None:
            for converter_name in utility.audio_formats:
                converter = query_utility(IMediaConverter, name=converter_name)
                if converter is not None:
                    yield converter_name, converter

    def has_conversion(self, converter):
        conversions = IMediaConversions(self.context)
        name = 'audio.{extension}'.format(extension=converter.format)
        return name in conversions
